import { useModalViewModel } from "@component/devices/view_model/modal.viewmodel";
import { useAppSelector } from "@hooks";
import React from "react";
import { Alert, Button, Form, Modal } from "react-bootstrap";
import { useTranslation } from "react-i18next";

interface AddDeviceModalProps {
  show: boolean;
  onHide: () => void;
}

const AddDeviceModal: React.FC<AddDeviceModalProps> = ({ show, onHide }) => {
  const { loading, register, handleSubmit, errors } = useModalViewModel({
    hide: onHide,
  });
  const { plugins } = useAppSelector((state) => state.plugin);
  const { t } = useTranslation("common");

  return (
    <Modal show={show} onHide={onHide} centered size="lg">
      <Modal.Header closeButton>
        <Modal.Title>{t("device.add_device.title")}</Modal.Title>
      </Modal.Header>
      <Modal.Body>
        {errors.root && (
          <Alert variant="danger" className="mt-3">
            {errors.root.message || t("device.add_device.error")}
          </Alert>
        )}
        <Form onSubmit={handleSubmit}>
          <Form.Group className="mb-3">
            <Form.Label>
              {t("device.add_device.fields.mne_name.label")}
            </Form.Label>
            <Form.Control
              type="text"
              placeholder={t("device.add_device.fields.mne_name.placeholder")}
              isInvalid={!!errors.mneName}
              {...register("mneName", {
                required: t("device.add_device.fields.mne_name.required"),
              })}
            />
            <Form.Control.Feedback type="invalid">
              {errors.mneName?.message}
            </Form.Control.Feedback>
          </Form.Group>
          <Form.Group className="mb-3">
            <Form.Label>
              {t("device.add_device.fields.address.label")}
            </Form.Label>
            <Form.Control
              type="text"
              placeholder={t("device.add_device.fields.address.placeholder")}
              isInvalid={!!errors.address}
              {...register("transportOption.address", {
                required: t("device.add_device.fields.address.required"),
              })}
            />
            <Form.Control.Feedback type="invalid">
              {errors.address?.message}
            </Form.Control.Feedback>
          </Form.Group>

          <Form.Group className="mb-3">
            <Form.Label>
              {t("device.add_device.fields.plugin.label")}
            </Form.Label>
            <Form.Control
              as="select"
              isInvalid={!!errors.pluginId}
              {...register("pluginId", {
                required: t("device.add_device.fields.plugin.required"),
              })}
            >
              <option value="">
                {t("device.add_device.fields.plugin.placeholder")}
              </option>
              {plugins.map((plugin) => (
                <option key={plugin.id} value={plugin.id}>
                  {plugin.manifest?.name}
                </option>
              ))}
            </Form.Control>
            <Form.Control.Feedback type="invalid">
              {errors.pluginId?.message}
            </Form.Control.Feedback>
          </Form.Group>

          <Form.Group className="mb-3">
            <Form.Check
              type="checkbox"
              label={t("device.add_device.fields.tls.label")}
              {...register("transportOption.tls")}
            />
          </Form.Group>

          <h5 className="mt-4">
            {t("device.add_device.fields.credentials.title")}
          </h5>
          <Form.Group className="mb-3">
            <Form.Label>
              {t("device.add_device.fields.credentials.username.label")}
            </Form.Label>
            <Form.Control
              type="text"
              placeholder={t(
                "device.add_device.fields.credentials.username.placeholder",
              )}
              isInvalid={!!errors.transportOption?.username}
              {...register("transportOption.username", {
                required: t(
                  "device.add_device.fields.credentials.username.required",
                ),
              })}
            />
            <Form.Control.Feedback type="invalid">
              {errors.transportOption?.username?.message}
            </Form.Control.Feedback>
          </Form.Group>

          <Form.Group className="mb-3">
            <Form.Label>
              {t("device.add_device.fields.credentials.password.label")}
            </Form.Label>
            <Form.Control
              type="password"
              placeholder={t(
                "device.add_device.fields.credentials.password.placeholder",
              )}
              isInvalid={!!errors.transportOption?.password}
              {...register("transportOption.password", {
                required: t(
                  "device.add_device.fields.credentials.password.required",
                ),
              })}
            />
            <Form.Control.Feedback type="invalid">
              {errors.transportOption?.password?.message}
            </Form.Control.Feedback>
          </Form.Group>

          <div className="d-flex justify-content-end gap-2">
            <Button variant="secondary" onClick={onHide}>
              {t("device.add_device.buttons.cancel")}
            </Button>
            <Button disabled={loading} variant="primary" type="submit">
              {loading ? (
                <>
                  <span className="spinner-border spinner-border-sm me-2" />
                  {t("device.add_device.buttons.loading")}
                </>
              ) : (
                t("device.add_device.buttons.submit")
              )}
            </Button>
          </div>
        </Form>
      </Modal.Body>
    </Modal>
  );
};
export default AddDeviceModal;
